''*****************************
''*    TV_WText 40x13 v1.2    *
''*      by Phil Pilgrim      *
''*     Based on TV_Text,     *
''*         which is          *
''*  (C) 2006 Parallax, Inc.  *
''*****************************

CON

  cols = 48
  rows = 24

  charBits  = 8
  scanlines = 8
  screensize = cols * rows
  linebits = charbits * cols
  screenbytes = cols * rows * scanlines
  screenwords = screenbytes/2
  screenlongs = cols / 2 * scanlines * rows
  screenlines = rows*scanlines

  videoPin = |< 12
  syncPin  = |< 6 
  genPin   = |< 13
  
VAR
  long  screen[12*192]
  
  LONG XPos          ' Current X position
  LONG YPos          ' Current Y position
  
  long  col, row,  flag
'  byte  eol, wleft, wtop, wcols, wrows
  byte  params[8],  pcnt, pptr

OBJ
   vga : "Video_Overlay384x192"
   
PUB start(basepin) : okay | temp 

'' Start terminal - starts a cog
'' returns false if no cog available
  
   okay := vga.start(@screen,videoPin,syncPin,genPin)
   out(0)
     
PUB stop

'' Stop terminal - frees a cog

  vga.stop


PUB str(stringptr)

'' Print a zero-terminated string

  repeat strsize(stringptr)
    out(byte[stringptr++])


PUB dec(value) | i

'' Print a decimal number

  if value < 0
    -value
    out("-")

  i := 1_000_000_000

  repeat 10
    if value => i
      out(value / i + "0")
      value //= i
      result~~
    elseif result or i == 1
      out("0")
    i /= 10


PUB hex(value, digits)

'' Print a hexadecimal number

  value <<= (8 - digits) << 2
  repeat digits
    out(lookupz((value <-= 4) & $F : "0".."9", "A".."F"))


PUB bin(value, digits)

'' Print a binary number

  value <<= 32 - digits
  repeat digits
    out((value <-= 1) & 1 + "0")

'' Display control codes
CON
  ClrWin      = $00                            ' clear window  
  Home        = $01                            ' home in window
  MoveXY      = $02                            ' move to X,Y in window (X and Y follow)
  CursLt      = $03                            ' cursor left
  CursRt      = $04                            ' cursor right
  CursUp      = $05                            ' cursor up
  CursDn      = $06                            ' cursor down
  Color       = $07                            ' select color C (0-63) (C follows)
  Bsp         = $08                            ' backspace
  Tab         = $09                            ' tab (8 spaces per)
  Lf          = $0A                            ' linefeed
  ClrEol      = $0B                            ' clear to end of line
  ClrEow      = $0C                            ' clear to end of active window
  Cr          = $0D                            ' carriage return
  MoveX       = $0E                            ' move to X in window (X follows)
  MoveY       = $0F                            ' move to Y in window (Y follows)
  SetWin      = $10                            ' set bounds of window (1-15,x,y,cols,rows)
                                               ' If cols or rows supplied as zero, use max. available
  UseWin      = $11                            ' change to selected window (0-15) (W follows)
  SetColor    = $12                            ' change color (0-63) to FG, BG (C, FG, BG follow)
                                               ' This actually changes a pair of colors for text use
                                               '  to FG,BG,FG,BG (even) and FG,FG,BG,BG (odd)
  WinColor    = $17                            ' change all colors in active window (C follows)
  EscChr      = $1F                            ' (ESC) print next character C as-is (C follows)
  Alt00       = $FF                            ' changed to $00 by out (for use in string())
  ClrScr      = Alt00                          ' equivalent to ClrWin (for use in string())
  
PUB out(c) | i, j, k

'' Output a character
''
''     $00 = clear active window
''     $01 = home in active window
''     $02 = move to X,Y in active window (X and Y follow)
''     $03 = cursor left
''     $04 = cursor right
''     $05 = cursor up
''     $06 = cursor down
''     $07 = select color C (0 - 15) (C follows)
''     $08 = backspace
''     $09 = tab (8 spaces per)
''     $0A = linefeed
''     $0B = clear to end of line
''     $0C = clear to end of active window
''     $0D = return
''     $0E = move to X in active window (X follows)
''     $0F = move to Y in active window (Y follows)
''     $10 = define window W (1 - 15) (W, Left, Top, nCols, nRows follow)
''     $11 = use window W (0 - 15) (W follows)
''     $12 = change color C (0 - 15) to FG, BG (C, FG, BG follow)
''     $17 = change all colors in active window to C (0 - 15) (C follows)
''     $1F = (ESC) print next character C as-is (C follows)
''  others = printable characters

  if flag
    if c == $FF
      c~
    params[pptr++] := c
    if pptr == pcnt
      case flag
        $02: col := params[0] <# cols - 1    'move to X,Y in active window (X and Y follow) 
             row := params[1] <# rows - 1
             Xpos := col*charBits
             Ypos := row*scanlines
             
        $07:     ' select color C (0 - 15) (C follows) 

        $0E: col := c <# cols - 1  ' move to X in active window (X follows) 
             Xpos := col*charBits
             
        $0F: row := c <# rows - 1   ' move to Y in active window (Y follows)                 
             Ypos := row*scanlines


        $1F: PutChar(c)  ' (ESC) print next character C as-is (C follows)
      flag~
                     
  else
    case c
      $00, $1E:      ' clear active window
          
             Cls(0)
           
      $01:     'home in active window
           
           Xpos := Ypos := 0
           
      $03:    ' move to X,Y in active window (X and Y follow)
           

      $04:   ' cursor right
           
      $05:     ' cursor up 

      $06:   ' cursor down
           

      $08:   ' backspace
'             if XPos >= Xpos*CharBits
             XPos -= CharBits

      $09: repeat    ' tab
             PutChar(" ")
           while col & 7
      
      $0A:    ' linefeed
             
             newline
             Xpos := 0
           
      $0B:  'cleartoeol   ' clear to end of line 
     
      $0C: 'i := row   ' clear to end of active window    
                      
     
      $0D:          ' carriage  return  
           newline
           Xpos := 0
      $13: 
             
      $14: 
       
      $15: 
       
      $16: 
       
      $07, $0E, $0F, $11, $17, $1F:
           flag := c
           pptr~
           pcnt := 1

      $02: flag := c
           pptr~
           pcnt := 2

      $12: flag := c
           pptr~
           pcnt := 3

      $10: flag := c
           pptr~
           pcnt := 5

      other: Putchar(c)


PUB Cls(Given)

  LONGFILL(@Screen, Given, 2304)
   
PUB MoveTo(X, Y)

  XPos:=X*8
  YPos:=Y*8 
      
PUB PutChar(Chr) | addr, i, shift, temp , char

  IF XPos+8 > 383
      newline
  char := chr
  shift:=8 - (XPos // 8)
  Chr:=(Chr-16)*8
  addr:=YPos*48 + XPos / 8
  REPEAT i FROM 0 TO 7
      if char == 32
        Screen.BYTE[addr]:=0
        Screen.BYTE[addr+1]:=0
      else
        temp:=CharData[Chr] << shift
        Screen.BYTE[addr]|=(temp.BYTE[1] >< 8)
        Screen.BYTE[addr+1]|=(temp.BYTE[0] >< 8)
    chr+=1
    addr+=48
  XPos+=8

PUB Plot(X, Y)
  X := 192 + x
  Y := 96 - y
  if x > 383
     x := 383
  if x < 0
     x := 0   
  if y > 191
     y := 191
  if y < 0
     y := 0   
  Screen.LONG[Y*12+X/32]:=Screen.LONG[Y*12+X/32] | (1 << (X // 32))

  
PRI newline
 
    XPos := 0
    YPos := Ypos + 8
    IF YPos => 192
'      LONGMOVE(@Screen, @Screen[64], 1472)
'      LONGFILL(@Screen[1471], 0, 64)
       LONGMOVE(@Screen, @Screen[96], 2208)
       LONGFILL(@Screen[2207], 0, 96)
      YPos:= 184
                             
DAT
CharData ' Very readable 8x8 font
  BYTE 0,0,0,0,0,0,0,0,240,240,240,240,0,0,0,0 ' 2x2 Black Blocks (8)
  BYTE 15,15,15,15,0,0,0,0,255,255,255,255,0,0,0,0
  BYTE 0,0,0,0,240,240,240,240,240,240,240,240,240,240,240,240
  BYTE 15,15,15,15,240,240,240,240,255,255,255,255,240,240,240,240
  BYTE 170,85,170,85,0,0,0,0,0,0,0,0,170,85,170,85 ' Gray blocks
  BYTE 170,85,170,85,170,85,170,85
  BYTE 1,3,7,15,31,63,127,255, 255,127,63,31,15,7,3,1 ' Diagonals (2)
  BYTE 60,66,129,129,66,36,231,0 ' Ohms Symbol
  BYTE 24,36,36,24,0,0,0,0 ' Degree Symbol
  BYTE 24,60,126,255,255,126,60,24 ' Dot
  BYTE  0,0,0,0,0,0,0,0,24,24,24,24,24,0,24,0 ' space !
  BYTE  102,102,102,0,0,0,0,0,108,108,254,108,254,108,108,0 ' "#
  BYTE  16,124,208,124,22,124,16,0,0,198,204,24,48,102,198,0 ' $%
  BYTE  56,108,56,118,220,204,118,0,24,24,48,0,0,0,0,0 ' &'
  BYTE  12,24,48,48,48,24,12,0,48,24,12,12,12,24,48,0 ' ()
  BYTE  0,108,56,254,56,108,0,0,0,24,24,126,24,24,0,0 ' *+
  BYTE  0,0,0,0,0,24,24,48,0,0,0,126,0,0,0,0 ' ,-
  BYTE  0,0,0,0,0,48,48,0,0,6,12,24,48,96,192,0 ' ./
  BYTE  60,102,110,126,118,102,60,0,12,28,60,12,12,12,12,0 '01
  BYTE  60,102,6,28,48,96,126,0,60,102,6,28,6,102,60,0 ' 23
  BYTE  28,60,108,204,254,12,12,0,126,96,96,124,6,102,60,0 ' 45
  BYTE  60,96,96,124,102,102,60,0,126,6,6,12,24,48,48,0 ' 67
  BYTE  60,102,102,60,102,102,60,0,60,102,102,62,6,6,60,0 ' 89
  BYTE  0,48,48,0,0,48,48,0,0,24,24,0,0,24,24,48 ' :;
  BYTE  12,24,48,96,48,24,12,0,0,0,126,0,0,126,0,0 ' <=
  BYTE  48,24,12,6,12,24,48,0,60,102,6,12,24,0,24,0 ' >?
  BYTE  60,102,110,110,108,96,60,0,24,60,102,102,126,102,102,0 ' @A
  BYTE  124,102,102,124,102,102,124,0,60,102,96,96,96,102,60,0 ' BC
  BYTE  124,102,102,102,102,102,124,0,126,96,96,124,96,96,126,0 ' DE
  BYTE  126,96,96,124,96,96,96,0,60,102,96,110,102,102,60,0 ' FG
  BYTE  102,102,102,126,102,102,102,0,60,24,24,24,24,24,60,0 ' HI
  BYTE  6,6,6,6,102,102,60,0,102,108,120,112,120,108,102,0 ' JK
  BYTE  96,96,96,96,96,96,126,0,198,238,254,214,198,198,198,0 ' LM
  BYTE  102,118,126,110,102,102,102,0,60,102,102,102,102,102,60,0 ' NO
  BYTE  124,102,102,124,96,96,96,0,60,102,102,102,102,110,60,6 ' PQ
  BYTE  124,102,102,124,102,102,102,0,60,102,96,60,6,102,60,0 ' RS
  BYTE  126,24,24,24,24,24,24,0,102,102,102,102,102,102,60,0 ' TU
  BYTE  102,102,102,102,102,60,24,0,198,198,198,214,254,238,198,0 ' VW
  BYTE  102,102,60,24,60,102,102,0,102,102,102,60,24,24,24,0 ' ZY
  BYTE  126,6,12,24,48,96,126,0,60,48,48,48,48,48,60,0 ' Z[
  BYTE  0,192,96,48,24,12,6,0,60,12,12,12,12,12,60,0 ' \]
  BYTE  24,60,102,0,0,0,0,0,0,0,0,0,0,0,0,255 ' ^_
  BYTE  24,24,12,0,0,0,0,0,0,0,60,6,62,102,62,0 ' `a
  BYTE  96,96,96,124,102,102,124,0,0,0,60,102,96,102,60,0 ' bc
  BYTE  6,6,6,62,102,102,62,0,0,0,60,102,126,96,62,0 ' de
  BYTE  28,54,48,124,48,48,48,0,0,0,62,102,102,62,6,60 ' fg
  BYTE  96,96,124,102,102,102,102,0,24,0,24,24,24,24,24,0 ' hi
  BYTE  12,0,12,12,12,12,108,56,96,96,102,108,120,108,102,0 ' jk
  BYTE  48,48,48,48,48,48,24,0,0,0,236,254,214,214,198,0 ' lm
  BYTE  0,0,124,102,102,102,102,0,0,0,60,102,102,102,60,0 ' no
  BYTE  0,0,124,102,102,124,96,96,0,0,62,102,102,62,6,6 ' pq
  BYTE  0,0,124,102,96,96,96,0,0,0,62,96,60,6,124,0 ' rs
  BYTE  48,48,124,48,48,54,28,0,0,0,102,102,102,102,62,0 ' tu
  BYTE  0,0,102,102,102,60,24,0,0,0,198,214,254,238,68,0 ' vw
  BYTE  0,0,102,60,24,60,102,0,0,0,102,102,102,62,6,60 ' xy
  BYTE  0,0,126,12,24,48,126,0,28,48,48,96,48,48,28,0 ' z{
  BYTE  24,24,24,24,24,24,24,0,56,12,12,6,12,12,56,0 ' |}
  BYTE  118,220,0,0,0,0,0,0,0,0,24,60,102,102,126,0 ' ~
                               